using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CodeTreeView
{
    public partial class FolderSelect : Form
    {
        private static string driveLetters = "CDEFG";
        public DirectoryInfo folder;
        public FolderSelect()
        {
            InitializeComponent();
            fillTree();
        }

        private void fillTree()
        {
            DirectoryInfo directory;
            string sCurPath = "";

            // clear out the old values
            treeView1.Nodes.Clear();

            // loop through the drive letters and find the available drives.
            foreach (char c in driveLetters)
            {
                sCurPath = c + ":\\";
                try
                {
                    // get the directory informaiton for this path.
                    directory = new DirectoryInfo(sCurPath);

                    // if the retrieved directory information points to a valid
                    // directory or drive in this case, add it to the root of the 
                    // treeView.
                    if (directory.Exists == true)
                    {
                        TreeNode newNode = new TreeNode(directory.FullName);
                        treeView1.Nodes.Add(newNode);	// add the new node to the root level.
                        getSubDirs(newNode);			// scan for any sub folders on this drive.
                    }
                }
                catch (Exception doh)
                {
                    Console.WriteLine(doh.Message);
                }
            }
        }

        /// <summary> method getSubDirs
        /// <para>this function will scan the specified parent for any subfolders 
        /// if they exist.  To minimize the memory usage, we only scan a single 
        /// folder level down from the existing, and only if it is needed.</para>
        /// <param name="parent">the parent folder in which to search for sub-folders.</param>
        /// </summary>
        private void getSubDirs(TreeNode parent)
        {
            DirectoryInfo directory;
            try
            {
                // if we have not scanned this folder before
                if (parent.Nodes.Count == 0)
                {
                    directory = new DirectoryInfo(parent.FullPath);
                    foreach (DirectoryInfo dir in directory.GetDirectories())
                    {
                        TreeNode newNode = new TreeNode(dir.Name);
                        parent.Nodes.Add(newNode);
                    }
                }

                // now that we have the children of the parent, see if they
                // have any child members that need to be scanned.  Scanning 
                // the first level of sub folders insures that you properly 
                // see the '+' or '-' expanding controls on each node that represents
                // a sub folder with it's own children.
                foreach (TreeNode node in parent.Nodes)
                {
                    // if we have not scanned this node before.
                    if (node.Nodes.Count == 0)
                    {
                        // get the folder information for the specified path.
                        directory = new DirectoryInfo(node.FullPath);

                        // check this folder for any possible sub-folders
                        foreach (DirectoryInfo dir in directory.GetDirectories())
                        {
                            // make a new TreeNode and add it to the treeView.
                            TreeNode newNode = new TreeNode(dir.Name);
                            node.Nodes.Add(newNode);
                        }
                    }
                }
            }
            catch (Exception doh)
            {
                Console.WriteLine(doh.Message);
            }
        }

        /// <summary> method fixPath
        /// <para>For some reason, the treeView will only work with paths constructed like the following example.
        /// "c:\\Program Files\Microsoft\...".  What this method does is strip the leading "\\" next to the drive
        /// letter.</para>
        /// <param name="node">the folder that needs it's path fixed for display.</param>
        /// <returns>The correctly formatted full path to the selected folder.</returns>
        /// </summary>
        private string fixPath(TreeNode node)
        {
            string sRet = "";
            try
            {
                sRet = node.FullPath;
                int index = sRet.IndexOf("\\\\");
                if (index > 1)
                {
                    sRet = node.FullPath.Remove(index, 1);
                }
            }
            catch (Exception doh)
            {
                Console.WriteLine(doh.Message);
            }
            return sRet;
        }
        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        //protected override void Dispose(bool disposing)
        //{
        //    if (disposing)
        //    {
        //        if (components != null)
        //        {
        //            components.Dispose();
        //        }
        //    }
        //    base.Dispose(disposing);
        //}


        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>

        /// <summary> method treeView1_BeforeSelect
        /// <para>Before we select a tree node we want to make sure that we scan the soon to be selected
        /// tree node for any sub-folders.  this insures proper tree construction on the fly.</para>
        /// <param name="sender">The object that invoked this event</param>
        /// <param name="e">The TreeViewCancelEventArgs event arguments.</param>
        /// <see cref="System.Windows.Forms.TreeViewCancelEventArgs"/>
        /// <see cref="System.Windows.Forms.TreeView"/>
        /// </summary>
        //private void treeView1_BeforeSelect(object sender, System.Windows.Forms.TreeViewCancelEventArgs e)
        //{
        //    getSubDirs(e.Node);					// get the sub-folders for the selected node.
        //    textBox1.Text = fixPath(e.Node);	// update the user selection text box.
        //    folder = new DirectoryInfo(e.Node.FullPath);	// get it's Directory info.
        //}

        /// <summary> method treeView1_BeforeExpand
        /// <para>Before we expand a tree node we want to make sure that we scan the soon to be expanded
        /// tree node for any sub-folders.  this insures proper tree construction on the fly.</para>
        /// <param name="sender">The object that invoked this event</param>
        /// <param name="e">The TreeViewCancelEventArgs event arguments.</param>
        /// <see cref="System.Windows.Forms.TreeViewCancelEventArgs"/>
        /// <see cref="System.Windows.Forms.TreeView"/>
        /// </summary>
        //private void treeView1_BeforeExpand(object sender, System.Windows.Forms.TreeViewCancelEventArgs e)
        //{
        //    getSubDirs(e.Node);					// get the sub-folders for the selected node.
        //    textBox1.Text = fixPath(e.Node);	// update the user selection text box.
        //    folder = new DirectoryInfo(e.Node.FullPath);	// get it's Directory info.
        //}

        /// <summary> method cancelBtn_Click
        /// <para>This method cancels the folder browsing.</para>
        /// </summary>
        private void cancelBtn_Click(object sender, System.EventArgs e)
        {
            folder = null;
            this.Close();
        }

        /// <summary> method selectBtn_Click
        /// <para>This method accepts which ever folder is selected and closes this application 
        /// with a DialogResult.OK result if you invoke this form though Form.ShowDialog().  
        /// In this example this method simply looks up the selected folder, and presents the 
        /// user with a message box displaying the name and path of the selected folder.
        // public browse b1;
        //private void selectBtn_Click(object sender, System.EventArgs e)
        //{
        //    try
        //    {
        //        // FolderSelTest fst = new FolderSelTest();

        //        //  fst.Show();
        //        this.DialogResult = DialogResult.OK;


        //        //this.Close();					
        //    }
        //    catch (Exception e2)
        //    {
        //        MessageBox.Show(e2.ToString());
        //    }
        //}

        //private void button5_Click(object sender, System.EventArgs e)
        //{
        //    Application.Exit();
        //}

        private void FolderSelect_Load(object sender, System.EventArgs e)
        {

        }
        /// <summary> method name
        /// <para>A method to retrieve the short name for the selected folder.</para>
        /// <returns>The folder name for the selected folder.</returns>
        /// </summary>
        public string name
        {
            get { return ((folder != null && folder.Exists)) ? folder.Name : null; }
        }

        /// <summary> method fullPath
        /// <para>Retrieve the full path for the selected folder.</para>
        /// 
        /// <returns>The correctly formatted full path to the selected folder.</returns>
        /// <seealso cref="FolderSelect.fixPath"/>
        /// </summary>
        public string fullPath
        {

            get { return ((folder != null && folder.Exists && treeView1.SelectedNode != null)) ? fixPath(treeView1.SelectedNode) : null; }
        }

        /// <summary> method info
        /// <para>Retrieve the full DirectoryInfo object associated with the selected folder.  Note
        /// that this will not have the corrected full path string.</para>
        /// <returns>The full DirectoryInfo object associated with the selected folder.</returns>
        /// <see cref="System.IO.DirectoryInfo"/>
        /// </summary>
        public DirectoryInfo info
        {
            get { return ((folder != null && folder.Exists)) ? folder : null; }
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            getSubDirs(e.Node);					// get the sub-folders for the selected node.
            textBox1.Text = fixPath(e.Node);	// update the user selection text box.
            folder = new DirectoryInfo(e.Node.FullPath);

        }

        private void selectBtn_Click_1(object sender, EventArgs e)
        {
            //try
            //{
            //    //browse dd = new browse();
            //    // dd.ShowDialog();
            //    // this.Close();

            //    //  fst.Show();
            //    this.DialogResult = DialogResult.OK;


            //    //this.Close();					
            //}
            //catch (Exception e2)
            //{
            //    MessageBox.Show(e2.ToString());
            //}

        }

        private void treeView1_BeforeSelect_1(object sender, TreeViewCancelEventArgs e)
        {
            //getSubDirs(e.Node);					// get the sub-folders for the selected node.
            //textBox1.Text = fixPath(e.Node);	// update the user selection text box.
            //folder = new DirectoryInfo(e.Node.FullPath);	// get it's Directory info.
        }

        private void treeView1_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            getSubDirs(e.Node);					// get the sub-folders for the selected node.
            textBox1.Text = fixPath(e.Node);	// update the user selection text box.
            folder = new DirectoryInfo(e.Node.FullPath);
        }

        private void button1_Click(object sender, EventArgs e)
        {

        }

        //private void selectBtn_Click(object sender, EventArgs e)
        //{

        //    try
        //    {
        //        //browse dd = new browse();
        //        // dd.ShowDialog();
        //        // this.Close();

        //        //  fst.Show();
        //        this.DialogResult = DialogResult.OK;


        //        //this.Close();					
        //    }
        //    catch (Exception e2)
        //    {
        //        MessageBox.Show(e2.ToString());
        //    }
        //}

        private void selectBtn_Click_2(object sender, EventArgs e)
        {
            try
            {
                //browse dd = new browse();
                // dd.ShowDialog();
                // this.Close();

                //  fst.Show();
                this.DialogResult = DialogResult.OK;


                //this.Close();					
            }
            catch (Exception e2)
            {
                MessageBox.Show(e2.ToString());
            }

        }

        private void cancelBtn_Click_1(object sender, EventArgs e)
        {
            folder = null;
            this.Close();

        }

        private void button5_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

       
    }
}